function Transition
% TRANSITION Generates simulations on the transitional dynamics for all models

stream0 = RandStream('mt19937ar','Seed',0);
stream1 = RandStream('mt19937ar','Seed',1);
RandStream.setDefaultStream(stream0);
RandStream.setDefaultStream(stream1);
load('models.mat');

options = struct(...
    'display'    , 1,...
    'reesolver'  , 'krylov',...
    'eqsolver'   , 'path',...
    'method'     , 'resapprox-complete',...
    'simulmethod', 'interpolation',...
    'stat'       , 0);

[gamma,Y,alpha,eta,D0,theta,v0,xi,mu,H0,k,delta,r] = without.model.params{:};
beta = 1/(1+r);

sext0 = [0 1 0 0];
s0 = [sum(sext0(1:3)) sext0(4)];

nrep = 1E5;
nper = 15;

%% Benchamrk model
stream0.reset
[ssim0,xsim0] = recsSimul(without.model,without.interp,repmat(s0(1),nrep,1), ...
                          50,[],options);

mx0  = squeeze(mean(xsim0,1))';
ms0  = squeeze(mean(ssim0,1))';
sdx0 = squeeze(std(xsim0,0,1))';
start = ssim0(:,:,end);
clear ssim0 xsim0

%% 2 instruments
stream1.reset
[ssim1,xsim1] = recsSimul(full.model,full.interp,[start zeros(nrep,1)],...
                          nper,[],options);

mx1  = squeeze(mean(xsim1,1))';
ms1  = squeeze(mean(ssim1,1))';
sdx1 = squeeze(std(xsim1,0,1))';
x1 = cat(3,zeros(nrep,9,1),xsim1);
clear ssim1 xsim1

%% Public storage only
stream1.reset
[ssim2,xsim2] = recsSimul(sto.model,sto.interp,[start zeros(nrep,1)],...
                          nper,[],options);

mx2  = squeeze(mean(xsim2,1))';
ms2  = squeeze(mean(ssim2,1))';
sdx2 = squeeze(std(xsim2,0,1))';
clear ssim2 xsim2

%% Tax/subsidy only
stream1.reset
[ssim3,xsim3] = recsSimul(sub.model,sub.interp,[start zeros(nrep,1)],...
                          nper,[],options);

mx3 = squeeze(mean(xsim3,1))';
ms3 = squeeze(mean(ssim3,1))';
sdx3 = squeeze(std(xsim3,0,1))';
clear ssim3 xsim3

CVx0 = sdx0./mx0;
CVx1 = sdx1./mx1;
CVx2 = sdx2./mx2;
CVx3 = sdx3./mx3;

CostS = (x1(:,3,2:end)+k).*x1(:,4,2:end)-x1(:,3,2:end).*x1(:,4,1:end-1)*(1-delta);
CostH = x1(:,6,2:end).*x1(:,2,2:end);
Cost  = CostS+CostH;
mCostS = [0 squeeze(mean(CostS,1))'];
mCostH = [0 squeeze(mean(CostH,1))'];
mCost  = [0 squeeze(mean(Cost,1))'];
meanCosts = [mCost' mCostS' mCostH'];

mS  = [[mx0(end,1); mx1(:,4)] [mx0(end,1); mx2(:,4)] [mx0(end,1); mx3(:,1)]];
mH  = [[mx0(end,2); mx1(:,2)] [mx0(end,2); mx2(:,2)] [mx0(end,2); mx3(:,2)]];
mP  = [[mx0(end,3); mx1(:,3)] [mx0(end,3); mx2(:,3)] [mx0(end,3); mx3(:,3)]];
CVP = [[CVx0(end,3); CVx1(:,3)] [CVx0(end,3); CVx2(:,3)] [CVx0(end,3); CVx3(:,3)]];

figure()
subplot(3,3,1)
plot(mS)
title('Stock')
subplot(3,3,2)
plot(-1:nper,mP)
xlim([-1 nper])
title('Mean price')
subplot(3,3,3)
plot(-1:nper,[CVx0(end,3); CVx1(:,3)],-1:nper,[CVx0(end,3); CVx2(:,3)],-1:nper,[CVx0(end,3); CVx3(:,3)])
xlim([-1 nper])
title('CV of price')
subplot(3,3,4)
plot(meanCosts)
title('Government costs')
subplot(3,3,5)
plot([mx0(end,2); mx1(:,2)])
title('Planned harvest')
subplot(3,3,6)
plot([ms0(end); ms1(:,1)])
title('Availability')
subplot(3,3,7)
plot([0; mx1(:,5)])
title('Subsidy/Tax')


% Welfare results
[W0,EPS0,~,PA0] = Welfmodel(without.model,without.interp,without.s,without.x,without.z,sext0(1:2));
[W1,EPS1,ECost1,~,EV1,ECostS1,ECostH1,Transfer1,PA1] = Welffull(full.model,full.interp,full.s,full.x,full.z,sext0);
[W2,EPS2,ECost2,~,EV2,Transfer2,PA2] = Welfsto(sto.model,sto.interp,sto.s,sto.x,sto.z,sext0);
[W3,EPS3,ECost3,~,EV3,Transfer3,PA3] = Welfsub(sub.model,sub.interp,sub.s,sub.x,sub.z,sext0([1:2 4]));
[W4,EPS4,ECost4,~,EV4,Transfer4,PA4] = Welffutures(futures.model, ...
                                                  futures.interp,futures.s, ...
                                                  futures.x,futures.z,sext0);

Welfres = zeros(7,4);
Welfres(:,1) = [EV1; EPS1-EPS0; PA1-PA0; ECost1; EV1+EPS1-ECost1-EPS0; Transfer1; EV1-Transfer1];
Welfres(:,2) = [EV2; EPS2-EPS0; PA2-PA0; ECost2; EV2+EPS2-ECost2-EPS0; Transfer2; EV2-Transfer2];
Welfres(:,3) = [EV3; EPS3-EPS0; PA3-PA0; ECost3; EV3+EPS3-ECost3-EPS0; Transfer3; EV3-Transfer3];
Welfres(:,4) = [EV4; EPS4-EPS0; PA4-PA0; ECost4; EV4+EPS4-ECost4-EPS0; Transfer4; EV4-Transfer4];

save('Transition.mat','mS','mP','mH','CVP','meanCosts','Welfres','-v6');

Welfres

NewOrder = [2:3 1 4];

fid = fopen('../TeX/Tables/OptSRWelf.tex','w');
fprintf(fid,'Consumers'' gains & %1.2f & %1.2f & %1.2f & %1.2f \\\\ \n',Welfres(1,NewOrder)*100);
fprintf(fid,'Producers'' gains & %1.2f & %1.2f & %1.2f & %1.2f \\\\ \n',Welfres(2,NewOrder)*100);
fprintf(fid,'\\quad from initial availability & %1.2f & %1.2f & %1.2f & %1.2f \\\\ \n',Welfres(3,NewOrder)*100);
fprintf(fid,['Government outlays & %1.2f & %1.2f & %1.2f & \\mc{$-$} \\\\ ' ...
             '\n'],Welfres(4,[2:3 1])*100);
fprintf(fid,'\\quad Public storage & %1.2f & \\mc{$-$} & %1.2f & \\mc{$-$} \\\\ \n',[Welfres(4,2) ECostS1]*100);
fprintf(fid,['\\quad Subsidy/tax to production & \\mc{$-$}  & %1.2f & %1.2f & ' ...
             '\\mc{$-$} \\\\ \n'],[Welfres(4,3) ECostH1]*100);
fprintf(fid,'Total gains & %1.2f & %1.2f & %1.2f & %1.2f \\\\ \n',Welfres(5,NewOrder)*100);
fwrite(fid,'\bottomrule');
fclose(fid);

disp('First-period cost as steady-state food budget share');
disp(meanCosts(2,1)*100/Y);
