function [out1,out2,out3,out4,out5] = mcommitmentfullbislinear(flag,s,x,z,e,snext,xnext, ...
                                                    gamma,Y,alpha,eta,D0,theta,Pbar, ...
                                                    w,mu,H0,k,delta,r)
% MCOMMITMENTFULLBISLINEAR Model definition: optimal policy with linear demand function

% For simplification this is an optimal policy without private storage (but
% private storage would be crowded out anyway). In addition, instead of using
% the social welfare function described in the paper, the objective here is a sum
% of surpluses to which is added a quadratic term in price to account for risk
% aversion. It is calibrated so that this is a second-order approximation to the
% true social welfare function (Turnovsky, Shalit & Schmitz, 1980, Econometrica).
  
n = size(s,1);
d = 1;
m = 5;
p = 4;

% s = A
jA       = 1;
if ~isempty(s)
  A       = s(:,jA);
end

% x = [Sg H P Xi Omega]
iSg     = 1;
iH      = 2;
iP      = 3;
iXi     = 4;
iOmega  = 5;


if ~isempty(x)
  Sg      = x(:,iSg);
  H       = x(:,iH);
  P       = x(:,iP);
  Xi      = x(:,iXi);
  Omega   = x(:,iOmega);
end

switch flag
 case 'f' % EQUILIBRIUM FUNCTION
   [D,dDdp] = demand(P,Y);
   
  % f
  out1            = zeros(n,m);
  out1(:,iSg)     = P+k+Omega-((1-delta)/(1+r))*z(:,3);
  out1(:,iH)      = -Xi+z(:,4)/(1+r);
  out1(:,iP)      = w*(P-Pbar)+Omega.*dDdp;
  out1(:,iXi)     = z(:,2)/(1+r)+Xi-H0*H.^mu;
  out1(:,iOmega)  = A-Sg-D;

  % df/dx
  if nargout>=2
    out2               = zeros(n,m,m);

    out2(:,iSg,iP)     = ones(n,1,1);
    out2(:,iSg,iOmega) = ones(n,1,1);

    out2(:,iH,iXi)     = -ones(n,1,1);

    out2(:,iP,iP)      = w*ones(n,1,1);
    out2(:,iP,iOmega)  = dDdp;
    
    out2(:,iXi,iH)  = -mu*H0*H.^(mu-1);
    out2(:,iXi,iXi) = ones(n,1,1);
    
    out2(:,iOmega,iSg) = -ones(n,1,1);
    out2(:,iOmega,iP)  = -dDdp;

    % df/dz
    if nargout>=3
      out3              = zeros(n,m,p);
      out3(:,iSg,3)     = -ones(n,1,1)*((1-delta)/(1+r));
      out3(:,iH,4)      = ones(n,1,1)/(1+r);
      out3(:,iXi,2)     = ones(n,1,1)/(1+r);
    end
  end
 case 'g' % STATE TRANSITION FUNCTION
  % g
  out1             = zeros(n,d);
  out1(:,jA)       = (1-delta)*Sg+H.*e;

  % dg/dx
  if nargout>=2
    out2                     = zeros(n,d,m);
    out2(:,jA,iSg)           = ones(n,1,1)*(1-delta);
    out2(:,jA,iH)            = e;

  end
 case 'h' % EXPECTATION FUNCTION
  % h
  out1      = zeros(size(snext,1),p);
  out1(:,1) = xnext(:,iP);
  out1(:,2) = xnext(:,iP);
  out1(:,3) = xnext(:,iP)+xnext(:,iOmega);
  out1(:,4) = xnext(:,iOmega);

   % dh/dx
  if nargout>=2
    out2 = zeros(size(snext,1),p,m);

    % dh/dsnext
    if nargout>=3
      out3 = zeros(size(snext,1),p,d);

      % dh/dxnext
      if nargout>=4
        out4             = zeros(size(snext,1),p,m);
        out4(:,1,iP)     = ones(size(snext,1),1,1);
        out4(:,2,iP)     = ones(size(snext,1),1,1);
        out4(:,3,iP)     = ones(size(snext,1),1,1);
        out4(:,3,iOmega) = ones(size(snext,1),1,1);
        out4(:,4,iOmega) = ones(size(snext,1),1,1);

        % hmult
        if nargout==5
          out5        = ones(size(snext,1),p);
          out5(:,2) = e;
          out5(:,4) = e;
        end
      end
    end
  end
 case 'b' % BOUND FUNCTION
  out1 = [zeros(n,1) -inf(n,m-1)];
  out2 = [A           inf(n,m-1)];
end


function [D,dDdp] = demand(p,y)

  b = -alpha;
  a = 1+b;
  D = a-b*p;
  dDdp = -b*ones(size(p));

end

end