function [model,interp,s,x,z] = runmodel(writeTex)
% RUNMODEL Run model: benchmark model without public policies

if nargin < 1, writeTex = 1; end

% GET parameters
[params,shocks,interp] = init_common;

[gamma,Y,alpha,eta,D0,theta,v0,w,mu,H0,k,delta,r] = params{:};
n = prod(interp.fspace.n);
s = gridmake(funnode(interp.fspace));

% PACK MODEL STRUCTURE
model.func   = 'mmodel';                               % model functions
model.params = params;               % other parameters
model        = catstruct(shocks,model);

xinit     = [max(zeros(n,1),0.8*(s-1)) ones(n,1) max((s/(D0*Y^eta)).^(1/alpha),0.7)];
interp.cz = ones(n,2);
interp.cx = xinit;
options = struct(...
    'display'    , 1,...
    'reesolver'  , 'mixed',...
    'eqsolver'   , 'path',...
    'method'     , 'resapprox-complete',...
    'simulmethod', 'interpolation',...
    'stat'       , 1,...
    'reesolveroptions',struct('atol',1E-10,'rtol',1E-10));

recsCheck(model,1,[0 1 1],[1 1]);

sext0 = [0 1];
s0    = sum(sext0);

tic
[interp.cx,x,z] = recsSolveREE(interp,model,s,xinit,options);
toc
interp.cz = funfitxy(interp.fspace,interp.Phi,z);

disp('Welfare weight calculated at the deterministic steady-state:');
disp(w);
vhat = Y^(1-eta)/(1-eta)-D0*x(:,3).^(1+alpha)/(1+alpha);
wt   = Y^(-eta).*vhat.^(theta)/v0;
w    = (1-1/(1+r))*infinitesums(model,interp,s,x,s0,wt);
disp('Welfare weight with a EV approach:');
disp(w);

stream0 = RandStream('mt19937ar','Seed',0);
RandStream.setDefaultStream(stream0);
stream0.reset
[ssim,xsim,esim] = recsSimul(model,interp,s0(ones(1E4,1),:),118,[],options);

disp('Welfare weight calculated on the asymptotic distribution:');
vhat = Y^(1-eta)/(1-eta)-D0*xsim(:,3,20:end).^(1+alpha)/(1+alpha);
w = mean(reshape(Y^(-eta).*vhat.^(theta)/v0,[],1));
disp(w);
model.params{8} = w;
save('Weight.mat','w');

recsAccuracy(model,interp,ssim);

[W0,EPS0,ESS0] = Welfmodel(model,interp,s,x,z,sext0)

vhat0 = Y^(1-eta)/(1-eta)-D0*1^(1+alpha)/(1+alpha);
rho   = eta-theta*Y^(1-eta)/vhat0;
EV = IdealEV(reshape(xsim(:,3,20:end),[],1),...
             rho,gamma,eta,alpha,...
             mean(reshape(xsim(:,3,20:end),[],1)));
disp('Equivalent variation from an ideal stabilisation:');
disp(EV);

without = struct('model',model,...
                 'interp',interp,...
                 's',s,...
                 'x',x,...
                 'z',z,...
                 'xsim',xsim);

save('models.mat','without');

% Extract rules for Mu and Chi
if (writeTex)
  A     = linspace(0.75,1.6,100)';
  [LB,UB] = feval(model.func,'b',A,[],[],[],[],[],params{:});
  [x,f] = recsSolveEquilibrium(A,...
                               min(max(funeval(interp.cx,interp.fspace,A),LB),UB),...
                               zeros(100,0),...
                               model.func,...
                               params,...
                               'path',...
                               interp.cx,model.e,model.w,interp.fspace,...
                               'resapprox-complete');
  mu = f(:,1);
  vhat = Y^(1-eta)/(1-eta)-D0*x(:,3).^(1+alpha)/(1+alpha);
  vy   = Y^(-eta).*vhat.^(theta)/v0;
  chi = (w-vy).*x(:,3)/alpha;
  save('Rules2.mat','A','mu','chi','-v6');

  % Plot risk-aversion with regard to price
  P = linspace(0.75,2,100)';
  vhat = Y^(1-eta)/(1-eta)-D0*P.^(1+alpha)/(1+alpha);
  rho = eta-theta*Y^(1-eta)./vhat;
  figure
  plot(P,rho);


  stream0.reset
  [~,xsim] = recsSimul(model,interp,s0,10000);

  save('sample.mat','xsim','-v6')
end
